@@ -36,13 +36,16 @@ import time
 import copy
 import ConfigParser
 from stat import *
+from utils import country_mirror, url_downloadable
 from string import Template
 
+
 import DistUpgradeView
 from DistUpgradeConfigParser import DistUpgradeConfig
 from DistUpgradeFetcherCore import country_mirror
 from DistUpgradeQuirks import DistUpgradeQuirks
 from DistUpgradeAptCdrom import AptCdrom
+from DistUpgradeAufs import setupAufs, aufsOptionsAndEnvironmentSetup
 
 from sourceslist import SourcesList, SourceEntry, is_mirror
 from distro import Distribution, get_distro, NoDistroTemplateException
@@ -54,9 +57,6 @@ import gettext
 from DistUpgradeCache import *
 from DistUpgradeApport import *
 
-# some constant
-# the initrd space required in /boot for each kernel
-KERNEL_INITRD_SIZE = 12*1024*1024
 
 class NoBackportsFoundException(Exception):
     pass
@@ -103,7 +103,10 @@ class DistUpgradeController(object):
         # ConfigParser deals only with strings it seems *sigh*
         self.config.add_section("Options")
         self.config.set("Options","withNetwork", str(self.useNetwork))
-        
+
+        # aufs stuff
+        aufsOptionsAndEnvironmentSetup(self.options, self.config)
+
         # some constants here
         self.fromDist = self.config.get("Sources","From")
         self.toDist = self.config.get("Sources","To")
@@ -124,6 +127,7 @@ class DistUpgradeController(object):
         # setup env var 
         os.environ["RELEASE_UPGRADE_IN_PROGRESS"] = "1"
         os.environ["PYCENTRAL_NO_DPKG_QUERY"] = "1"
+        os.environ["PYCENTRAL_FORCE_OVERWRITE"] = "1"
         os.environ["PATH"] = "%s:%s" % (os.getcwd()+"/imported",
                                         os.environ["PATH"])
 
@@ -158,6 +162,8 @@ class DistUpgradeController(object):
                                  self.quirks,
                                  self._view.getOpCacheProgress(),
                                  lock)
+            # alias name for the plugin interface code
+            self.apt_cache = self.cache
         # if we get a dpkg error that it was interrupted, just
         # run dpkg --configure -a
         except CacheExceptionDpkgInterrupted, e:
@@ -176,8 +182,42 @@ class DistUpgradeController(object):
                                "already running. Please close that "
                                "application first."));
             sys.exit(1)
+        self.cache.partialUpgrade = self._partialUpgrade
         logging.debug("/openCache()")
 
+    def _isRemoteLogin(self):
+        " check if we are running form a remote login "
+        # easy
+        if (os.environ.has_key("SSH_CONNECTION") or
+            os.environ.has_key("SSH_TTY")):
+            return True
+        # sudo cleans out SSH_ environment
+        out = subprocess.Popen(["who","-m","--ips"],stdout=subprocess.PIPE).communicate()[0]
+        logging.debug("who -m --ips: '%s'" % out)
+        # the host is in ()
+        if not "(" in out:
+            return False
+        # if we have a () parse it
+        ip = out.strip().rsplit('(')[1]
+        ip = ip.strip(')')
+        # if we have a ip here and it does not start with a
+        # ":" we have a remote login
+        # FIXME: what about IPv6 ?
+        if not ip.startswith(":"):
+            return True
+        return False
+
+    def _viewSupportsSSH(self):
+      """
+      Returns True if this view support upgrades over ssh.
+      In theory all views should support it, but for savety
+      we do only allow text ssh upgrades (see LP: #322482)
+      """
+      supported = self.config.getlist("View","SupportSSH")
+      if self._view.__class__.__name__ in supported:
+          return True
+      return False
+
     def _sshMagic(self):
         """ this will check for server mode and if we run over ssh.
             if this is the case, we will ask and spawn a additional
@@ -185,9 +225,20 @@ class DistUpgradeController(object):
             of trouble)
         """
         pidfile = os.path.join("/var/run/release-upgrader-sshd.pid")
-        if (not os.path.exists(pidfile) and
-            (os.environ.has_key("SSH_CONNECTION") or
-             os.environ.has_key("SSH_TTY"))):
+        if (not os.path.exists(pidfile) and self._isRemoteLogin()):
+            # check if the frontend supports ssh ugprades (see lp: #322482)
+            if not self._viewSupportsSSH():
+                logging.error("upgrade over ssh not alllowed")
+                self._view.error(_("Upgrading over remote connection not supported"),
+                                 _("You are running the upgrade over a "
+                                   "remote ssh connection with a frontend "
+                                   "that does "
+                                   "not support this. The upgrade will "
+                                   "abort now. Please try without ssh.")
+                                 )
+                sys.exit(1)
+                return False
+            # ask for a spare one to start
             port = 9004
             res = self._view.askYesNoQuestion(
                 _("Continue running under SSH?"),
@@ -212,6 +263,7 @@ class DistUpgradeController(object):
                       "If anything goes wrong with the running ssh "
                       "you can still connect to the additional one.\n"
                       ) % port)
+        return True
 
     def _tryUpdateSelf(self):
         """ this is a helper that is run if we are started from a CD
@@ -266,7 +318,8 @@ class DistUpgradeController(object):
                 logging.debug("python symlink points to: '%s', but expected is '%s' or '%s'" % (fs_default_version, expected_default, os.path.join('/usr/bin', expected_default)))
                 return False
         return True
-    
+
+
     def prepare(self):
         """ initial cache opening, sanity checking, network checking """
         # first check if that is a good upgrade
@@ -279,6 +332,28 @@ class DistUpgradeController(object):
                              _("An upgrade from '%s' to '%s' is not "
                                "supported with this tool." % (release, self.toDist)))
             sys.exit(1)
+
+        # setup aufs
+        if self.config.getWithDefault("Aufs", "EnableFullOverlay", False):
+            aufs_rw_dir = self.config.get("Aufs","RWDir")
+            if not setupAufs(aufs_rw_dir):
+                logging.error("aufs setup failed")
+                self._view.error(_("Sandbox setup failed"),
+                                 _("It was not possible to create the sandbox "
+                                   "environment."))
+                return False
+
+            # all good, tell the user about the sandbox mode
+            logging.info("running in aufs overlay mode")
+            self._view.information(_("Sandbox mode"),
+                                   _("This upgrade is running in sandbox "
+                                     "(test) mode. All changes are written "
+                                     "to '%s' and will be lost on the next "
+                                     "reboot.\n\n"
+                                     "*No* changes written to a systemdir "
+                                     "from now until the next reboot are "
+                                     "permanent.") % aufs_rw_dir)
+
         # setup backports (if we have them)
         if self.options and self.options.havePrerequists:
             backportsdir = os.getcwd()+"/backports"
@@ -357,9 +432,32 @@ class DistUpgradeController(object):
                 self._tryUpdateSelf()
         return True
 
+    def _sourcesListEntryDownloadable(self, entry):
+        """
+        helper that checks if a sources.list entry points to 
+        something downloadable
+        """
+        logging.debug("verifySourcesListEntry: %s" % entry)
+        # no way to verify without network
+        if not self.useNetwork:
+            logging.debug("skiping downloadable check (no network)")
+            return True
+        # check if the entry points to something we can download
+        uri = "%s/dists/%s/Release" % (entry.uri, entry.dist)
+        return url_downloadable(uri, logging.debug)
+
     def rewriteSourcesList(self, mirror_check=True):
         logging.debug("rewriteSourcesList()")
 
+        sync_components = self.config.getlist("Sources","Components")
+
+        # skip mirror check if special environment is set
+        # (useful for server admins with internal repos)
+        if (self.config.getWithDefault("Sources","AllowThirdParty",False) or
+            "RELEASE_UPRADER_ALLOW_THIRD_PARTY" in os.environ):
+            logging.warning("mirror check skipped, *overriden* via config")
+            mirror_check=False
+
         # check if we need to enable main
         if mirror_check == True and self.useNetwork:
             # now check if the base-meta pkgs are available in
@@ -384,7 +482,10 @@ class DistUpgradeController(object):
                         # we replace the sources.list with a single
                         # line to ubuntu-main
                         logging.warning('get_distro().enable_component("man") failed, overwriting sources.list instead as last resort')
-                        s = "deb http://archive.ubuntu.com/ubuntu %s main" % self.toDist
+                        s =  "# auto generated by update-manager"
+                        s += "deb http://archive.ubuntu.com/ubuntu %s main restricted" % self.toDist
+                        s += "deb http://archive.ubuntu.com/ubuntu %s-updates main restricted" % self.toDist
+                        s += "deb http://security.ubuntu.com/ubuntu %s-security main restricted" % self.toDist
                         open("/etc/apt/sources.list","w").write(s)
                     break
             
@@ -408,6 +509,9 @@ class DistUpgradeController(object):
 
         # look over the stuff we have
         foundToDist = False
+        # collect information on what components (main,universe) are enabled for what distro (sub)version
+        # e.g. found_components = { 'hardy':set("main","restricted"), 'hardy-updates':set("main") }
+        found_components = {}
         for entry in self.sources.list[:]:
 
             # ignore invalid records or disabled ones
@@ -435,22 +539,28 @@ class DistUpgradeController(object):
                 continue
 
             # special case for landscape.canonical.com because they
-            # don't use a standard archive layout
+            # don't use a standard archive layout (gutsy->hardy)
             if (not entry.disabled and
                 entry.uri.startswith("http://landscape.canonical.com/packages/%s" % self.fromDist)):
                 logging.debug("commenting landscape.canonical.com out")
                 entry.disabled = True
                 continue
-
-            # special case for old-releases.ubuntu.com, auto transition
-            # them back to archive.ubuntu.com - now this is a problem
-            # of course for people upgrading from EOL release to a 
-            # EOL release
+            
+            # handle upgrades from a EOL release and check if there
+            # is a supported release available
             if (not entry.disabled and
-                entry.uri.startswith("http://old-releases.ubuntu.com/ubuntu")):
-                entry.uri = "http://archive.ubuntu.com/ubuntu"
-                logging.debug("transitioning old-releases.ubuntu.com to '%s' " % entry)
-                continue
+                "old-releases.ubuntu.com/" in entry.uri):
+                logging.debug("upgrade from old-releases.ubuntu.com detected")
+                # test country mirror first, then archive.u.c
+                for uri in ["http://%sarchive.ubuntu.com/ubuntu" % country_mirror(),
+                            "http://archive.ubuntu.com/ubuntu"]:
+                    test_entry = copy.copy(entry)
+                    test_entry.uri = uri
+                    test_entry.dist = self.toDist
+                    if self._sourcesListEntryDownloadable(test_entry):
+                        logging.info("transition from old-release.u.c to %s" % uri)
+                        entry.uri = uri
+                        break
 
             logging.debug("examining: '%s'" % entry)
             # check if it's a mirror (or official site)
@@ -482,25 +592,56 @@ class DistUpgradeController(object):
                     entry.disabled = True
                     self.sources_disabled = True
                     logging.debug("entry '%s' was disabled (unknown dist)" % entry)
-                
+
                 # if we make it to this point, we have a official mirror
-                # 
+
                 # check if the arch is powerpc or sparc and if so, transition
                 # to ports.ubuntu.com (powerpc got demoted in gutsy, sparc
                 # in hardy)
-                    
                 if (entry.type == "deb" and
                     not "ports.ubuntu.com" in entry.uri and
                     (self.arch == "powerpc" or self.arch == "sparc")):
                     logging.debug("moving %s source entry to 'ports.ubuntu.com' " % self.arch)
                     entry.uri = "http://ports.ubuntu.com/ubuntu-ports/"
-                    
 
+                # gather what components are enabled and are inconsitent
+                for d in ["%s" % self.toDist,
+                          "%s-updates" % self.toDist,
+                          "%s-security" % self.toDist]:
+                    # create entry if needed, ignore disabled
+                    # entries and deb-src
+                    found_components.setdefault(d,set())
+                    if (not entry.disabled and entry.dist == d and
+                        entry.type == "deb"):
+                        for comp in entry.comps:
+                            # only sync components we know about
+                            if not comp in sync_components:
+                                continue
+                            found_components[d].add(comp)
+                    
             # disable anything that is not from a official mirror
             if not validMirror:
+                if entry.dist == self.fromDist:
+                    entry.dist = self.toDist
+                entry.comment += " " + _("disabled on upgrade to %s") % self.toDist
                 entry.disabled = True
                 self.sources_disabled = True
                 logging.debug("entry '%s' was disabled (unknown mirror)" % entry)
+
+        # now go over the list again and check for missing components 
+        # in $dist-updates and $dist-security and add them
+        for entry in self.sources.list[:]:
+            # skip all comps that are not relevant (including e.g. "hardy")
+            if (entry.invalid or entry.disabled or entry.type == "deb-src" or 
+                entry.uri.startswith("cdrom:") or entry.dist == self.toDist):
+                continue
+            # now check for "$dist-updates" and "$dist-security" and add any inconsistencies
+            if found_components.has_key(entry.dist):
+                # add the delta between "hardy" comps and "hardy-updates" comps once
+                logging.info("fixing components inconsistency from '%s'" % entry)
+                entry.comps.extend(list(found_components[self.toDist]-found_components[entry.dist]))
+                logging.info("to new entry '%s'" % entry)
+                del found_components[entry.dist]
         return foundToDist
 
     def updateSourcesList(self):
@@ -540,6 +681,7 @@ class DistUpgradeController(object):
 
                     # add some defaults here
                     # FIXME: find mirror here
+                    logging.info("generate new default sources.list")
                     uri = "http://archive.ubuntu.com/ubuntu"
                     comps = ["main","restricted"]
                     self.sources.add("deb", uri, self.toDist, comps)
@@ -605,7 +747,8 @@ class DistUpgradeController(object):
         # check if we have packages in ReqReinst state that are not
         # downloadable
         logging.debug("doPostInitialUpdate")
-        self.quirks.run("PostInitialUpdate")
+        if not self._partialUpgrade:
+            self.quirks.run("PostInitialUpdate")
         if len(self.cache.reqReinstallPkgs) > 0:
             logging.warning("packages in reqReinstall state, trying to fix")
             self.cache.fixReqReinst(self._view)
@@ -622,7 +765,7 @@ class DistUpgradeController(object):
                                "or remove it from the system.",
                                "The packages '%s' are in an inconsistent "
                                "state and need to be reinstalled, but "
-                               "no archive can be found for it."
+                               "no archive can be found for them. "
                                "Please reinstall the packages manually "
                                "or remove them from the system.",
                                len(reqreinst)) % ", ".join(reqreinst)
@@ -674,101 +817,30 @@ class DistUpgradeController(object):
     def _checkFreeSpace(self):
         " this checks if we have enough free space on /var and /usr"
         err_sum = _("Not enough free disk space")
-        err_long= _("The upgrade aborts now. "
+        err_long= _("The upgrade is now aborted. "
                     "The upgrade needs a total of %s free space on disk '%s'. "
                     "Please free at least an additional %s of disk "
                     "space on '%s'. "
                     "Empty your trash and remove temporary "
                     "packages of former installations using "
                     "'sudo apt-get clean'.")
-
-        class FreeSpace(object):
-            " helper class that represents the free space on each mounted fs "
-            def __init__(self, initialFree):
-                self.free = initialFree
-                self.need = 0
-
-        def make_fs_id(d):
-            """ return 'id' of a directory so that directories on the
-                same filesystem get the same id (simply the mount_point)
-            """
-            for mount_point in mounted:
-                if d.startswith(mount_point):
-                    return mount_point
-            return "/"
-
-        # this is all a bit complicated
-        # 1) check what is mounted (in mounted)
-        # 2) create FreeSpace objects for the dirs we are interested in
-        #    (mnt_map)
-        # 3) use the  mnt_map to check if we have enough free space and
-        #    if not tell the user how much is missing
-        mounted = []
-        mnt_map = {}
-        fs_free = {}
-        for line in open("/proc/mounts"):
-            try:
-                (what, where, fs, options, a, b) = line.split()
-            except ValueError, e:
-                logging.debug("line '%s' in /proc/mounts not understood (%s)" % (line, e))
-                continue
-            if not where in mounted:
-                mounted.append(where)
-        # make sure mounted is sorted by longest path
-        mounted.sort(cmp=lambda a,b: cmp(len(a),len(b)), reverse=True)
-        archivedir = apt_pkg.Config.FindDir("Dir::Cache::archives")
-        for d in ["/","/usr","/var","/boot", archivedir, "/home"]:
-            d = os.path.realpath(d)
-            fs_id = make_fs_id(d)
-            st = os.statvfs(d)
-            free = st[statvfs.F_BAVAIL]*st[statvfs.F_FRSIZE]
-            if fs_id in mnt_map:
-                logging.debug("Dir %s mounted on %s" % (d,mnt_map[fs_id]))
-                fs_free[d] = fs_free[mnt_map[fs_id]]
-            else:
-                logging.debug("Free space on %s: %s" % (d,free))
-                mnt_map[fs_id] = d
-                fs_free[d] = FreeSpace(free)
-        del mnt_map
-        logging.debug("fs_free contains: '%s'" % fs_free)
-
-        # now calculate the space that is required on /boot
-        # we do this by checking how many linux-image-$ver packages
-        # are installed or going to be installed
-        space_in_boot = 0
-        for pkg in self.cache:
-            # we match against everything that looks like a kernel
-            # and add space check to filter out metapackages
-            if re.match("^linux-(image|image-debug)-[0-9.]*-.*", pkg.name):
-                if pkg.markedInstall:
-                    logging.debug("%s (new-install) added with %s to boot space" % (pkg.name, KERNEL_INITRD_SIZE))
-                    space_in_boot += KERNEL_INITRD_SIZE
-                elif (pkg.markedUpgrade or pkg.isInstalled):
-                    logging.debug("%s (upgrade|installed) added with %s to boot space" % (pkg.name, KERNEL_INITRD_SIZE))
-                    space_in_boot += KERNEL_INITRD_SIZE # creates .bak
-
-        # we check for various sizes:
-        # archivedir is were we download the debs
-        # /usr is assumed to get *all* of the install space (incorrect,
-        #      but as good as we can do currently + safety buffer
-        # /     has a small safety buffer as well
-        for (dir, size) in [(archivedir, self.cache.requiredDownload),
-                            ("/usr", self.cache.additionalRequiredSpace),
-                            ("/usr", 50*1024*1024),  # safety buffer /usr
-                            ("/boot", space_in_boot), 
-                            ("/", 10*1024*1024),     # small safety buffer /
-                           ]:
-            dir = os.path.realpath(dir)
-            logging.debug("dir '%s' needs '%s' of '%s' (%f)" % (dir, size, fs_free[dir], fs_free[dir].free))
-            fs_free[dir].free -= size
-            fs_free[dir].need += size
-            if fs_free[dir].free < 0:
-                free_at_least = apt_pkg.SizeToStr(float(abs(fs_free[dir].free)+1))
-                logging.error("not enough free space on %s (missing %s)" % (dir, free_at_least))
-                self._view.error(err_sum, err_long % (apt_pkg.SizeToStr(fs_free[dir].need), make_fs_id(dir), free_at_least, make_fs_id(dir)))
-                return False
-
-            
+        # allow override
+        if self.config.getWithDefault("FreeSpace","SkipCheck",False):
+            logging.warning("free space check skipped via config override")
+            return True
+        # do the check
+        try:
+            self.cache.checkFreeSpace()
+        except NotEnoughFreeSpaceError, e:
+            # ok, showing multiple error dialog sucks from the UI
+            # perspective, but it means we do not need to break the
+            # string freeze
+            for required in e.free_space_required_list:
+                self._view.error(err_sum, err_long % (required.size_total,
+                                                      required.dir,
+                                                      required.size_needed,
+                                                      required.dir))
+            return False
         return True
 
 
@@ -863,7 +935,7 @@ class DistUpgradeController(object):
         # maximum fetch-retries reached without a successful commit
         logging.error("giving up on fetching after maximum retries")
         self._view.error(_("Could not download the upgrades"),
-                         _("The upgrade aborts now. Please check your "
+                         _("The upgrade is now aborted. Please check your "
                            "Internet connection or "
                            "installation media and try again. All files "
                            "downloaded so far are kept."),
@@ -871,36 +943,66 @@ class DistUpgradeController(object):
         # abort here because we want our sources.list back
         self._enableAptCronJob()
         self.abort()
+
+    def enableApport(self, fname="/etc/default/apport"):
+        " enable apoprt "
+        # for jaunty and later we could use this instead:
+        #  env = copy.copy(os.environ)
+        #  env["force_start"] = "1"
+        #  subprocess.call(["/etc/init.d/apport","start"], env=env)
+        # but hardy and intrepid do not have the force_start yet
+        if not os.path.exists(fname):
+            return
+        # copy the jaunty version of the conf file in place
+        # (this avoids a conffile prompt later)
+        logging.debug("enabling apport")
+        shutil.copy("etc-default-apport","/etc/default/apport")
+        subprocess.call(["/etc/init.d/apport","start"])
         
-    
     def doDistUpgrade(self):
+        # check if we want apport running during the upgrade
+        if self.config.getWithDefault("Distro","EnableApport", False):
+            self.enableApport()
         # get the upgrade
         currentRetry = 0
         fprogress = self._view.getFetchProgress()
         iprogress = self._view.getInstallProgress(self.cache)
         # retry the fetching in case of errors
         maxRetries = self.config.getint("Network","MaxRetries")
+        self.quirks.run("StartUpgrade")
         while currentRetry < maxRetries:
             try:
                 res = self.cache.commit(fprogress,iprogress)
             except SystemError, e:
                 logging.error("SystemError from cache.commit(): %s" % e)
-                # check if the installprogress catched a pkg_failures, 
-                # if not, generate a fallback here (pkg_failures can be
-                # zero because of e.g. disk-full problems)
-                if iprogress.pkg_failures == 0:
-                    logging.warning("cache.commit() error and pkg_failures == 0, generate a report against update-manager to investigate")
-                    errormsg = "SystemError in cache.commit(): %s" % e
-                    apport_pkgfailure("update-manager", errormsg)
-                # invoke the frontend now
-                msg = _("The upgrade aborts now. Your system "
+                # if its a ordering bug we can cleanly revert to
+                # the previous release, no packages have been installed
+                # yet (LP: #328655, #356781)
+                pre_configure_errors = [
+                  "E:Internal Error, Could not perform immediate configuration",
+                  "E:Couldn't configure pre-depend "]
+                for preconf_error in pre_configure_errors:
+                    if str(e).startswith(preconf_error):
+                        logging.debug("detected preconfigure error, restorting state")
+                        self._enableAptCronJob()
+                        # FIXME: strings are not good, but we are in string freeze
+                        # currently
+                        msg = _("Error during commit")
+                        msg += "\n'%s'\n" % str(e)
+                        msg += _("Restoring original system state")
+                        self._view.error(_("Could not install the upgrades"), msg)
+                        # abort() exits cleanly
+                        self.abort()
+                
+                # invoke the frontend now and show a error message
+                msg = _("The upgrade is now aborted. Your system "
                         "could be in an unusable state. A recovery "
                         "will run now (dpkg --configure -a).")
                 if not self._partialUpgrade:
                     if not run_apport():
                         msg += _("\n\nPlease report this bug against the 'update-manager' "
                                  "package and include the files in /var/log/dist-upgrade/ "
-                                 "in the bugreport.\n"
+                                 "in the bug report.\n"
                                  "%s" % e)
                 self._view.error(_("Could not install the upgrades"), msg)
                 # installing the packages failed, can't be retried
@@ -919,7 +1021,7 @@ class DistUpgradeController(object):
         # maximum fetch-retries reached without a successful commit
         logging.error("giving up on fetching after maximum retries")
         self._view.error(_("Could not download the upgrades"),
-                         _("The upgrade aborts now. Please check your "\
+                         _("The upgrade is now aborted. Please check your "\
                            "Internet connection or "\
                            "installation media and try again. "),
                            "%s" % e)
@@ -940,6 +1042,14 @@ class DistUpgradeController(object):
         now_foreign = self.cache._getForeignPkgs(self.origin, self.fromDist, self.toDist)
         logging.debug("Obsolete: %s" % " ".join(now_obsolete))
         logging.debug("Foreign: %s" % " ".join(now_foreign))
+        # now sanity check - if a base meta package is in the obsolete list now, that means
+        # that something went wrong (see #335154) badly with the network. this should never happen, but it did happen
+        # at least once so we add extra paranoia here
+        for pkg in self.config.getlist("Distro","BaseMetaPkgs"):
+            if pkg in now_obsolete:
+                logging.error("the BaseMetaPkg '%s' is in the obsolete list, something is wrong, ignoring the obsoletes" % pkg)
+                now_obsolete = set()
+                break
         # check if we actually want obsolete removal
         if not self.config.getWithDefault("Distro","RemoveObsoletes", True):
             logging.debug("Skipping obsolete Removal")
@@ -1006,6 +1116,9 @@ class DistUpgradeController(object):
                                    "Please see the below message for more "
                                    "information. "),
                                    "%s" % e)
+        # run stuff after cleanup
+        if not self._partialUpgrade:
+            self.quirks.run("PostCleanup")
         # run the post upgrade scripts that can do fixup like xorg.conf
         # fixes etc - only do on real upgrades
         if not self._partialUpgrade:
@@ -1209,6 +1322,10 @@ class DistUpgradeController(object):
         # 
         # make sure to remove file on cancel
         
+        # FIXME: use the DistUpgradeFetcherCore logic
+        #        in mirror_from_sources_list() here
+        #        (and factor that code out into a helper)
+
         conf_option = "SourcesList"
         if self.config.has_option("PreRequists",conf_option+"-%s" % self.arch):
             conf_option = conf_option + "-%s" % self.arch
@@ -1337,7 +1454,7 @@ class DistUpgradeController(object):
                                "against the 'update-manager' "
                                "package and include the files in "
                                "/var/log/dist-upgrade/ "
-                               "in the bugreport." ))
+                               "in the bug report." ))
             sys.exit(1)
 
         # mvo: commented out for now, see #54234, this needs to be
@@ -1358,7 +1475,7 @@ class DistUpgradeController(object):
                                    "against the 'update-manager' "
                                    "package and include the files in "
                                    "/var/log/dist-upgrade/ "
-                                   "in the bugreport." ))
+                                   "in the bug report." ))
                 self.abort()
 
         # run a "apt-get update" now, its ok to ignore errors, 
@@ -1403,7 +1520,7 @@ class DistUpgradeController(object):
                 #        but we need to be careful to not duplicate them
                 #        (i.e. the error here could be something else than
                 #        missing sources entries but network errors etc)
-                logging.error("No '%s' after sources.list rewrite+update")
+                logging.error("No '%s' after sources.list rewrite+update" % pkg) 
                 self._view.error(_("Invalid package information"),
                                  _("After your package information was "
                                    "updated the essential package '%s' can "
@@ -1411,7 +1528,7 @@ class DistUpgradeController(object):
                                    "This indicates a serious error, please "
                                    "report this bug against the 'update-manager' "
                                    "package and include the files in /var/log/dist-upgrade/ "
-                                   "in the bugreport.") % pkg)
+                                   "in the bug report.") % pkg)
                 self.abort()
 
         # calc the dist-upgrade and see if the removals are ok/expected
@@ -1436,7 +1553,7 @@ class DistUpgradeController(object):
             # don't abort here, because it would restore the sources.list
             self._view.information(_("Upgrade complete"),
                                    _("The upgrade is completed but there "
-                                     "were errors during the ugprade "
+                                     "were errors during the upgrade "
                                      "process."))
             sys.exit(1) 
             
@@ -1478,14 +1595,14 @@ class DistUpgradeController(object):
         if not self.doDistUpgrade():
             self._view.information(_("Upgrade complete"),
                                    _("The upgrade is completed but there "
-                                     "were errors during the ugprade "
+                                     "were errors during the upgrade "
                                      "process."))
             return False
         self._view.setStep(STEP_CLEANUP)
         if not self.doPostUpgrade():
             self._view.information(_("Upgrade complete"),
                                    _("The upgrade is completed but there "
-                                     "were errors during the ugprade "
+                                     "were errors during the upgrade "
                                      "process."))
             return False
         self._view.information(_("Upgrade complete"),
