@@ -33,6 +33,7 @@ try:
 except:
     import fakegconf as gconf
 import gobject
+
 import warnings
 warnings.filterwarnings("ignore", "apt API not stable yet", FutureWarning)
 import apt
@@ -57,22 +58,26 @@ import socket
 import time
 import thread
 import xml.sax.saxutils
-from Common.HelpViewer import HelpViewer
 
 import dbus
 import dbus.service
 import dbus.glib
 
+import GtkProgress
+
 from gettext import gettext as _
 from gettext import ngettext
 
-from Common.utils import *
-from Common.SimpleGladeApp import SimpleGladeApp
+
+from Core.utils import *
+from Core.UpdateList import UpdateList
+from Core.MyCache import MyCache, NotEnoughFreeSpaceError
+from Core.MetaRelease import Dist
+
 from DistUpgradeFetcher import DistUpgradeFetcherGtk
 from ChangelogViewer import ChangelogViewer
-import GtkProgress
-
-from Core.MetaRelease import Dist
+from SimpleGladeApp import SimpleGladeApp
+from HelpViewer import HelpViewer
 from MetaReleaseGObject import MetaRelease
 
 #import pdb
@@ -86,241 +91,6 @@ from MetaReleaseGObject import MetaRelease
 # actions for "invoke_manager"
 (INSTALL, UPDATE) = range(2)
 
-SYNAPTIC_PINFILE = "/var/lib/synaptic/preferences"
-
-CHANGELOGS_URI="http://changelogs.ubuntu.com/changelogs/pool/%s/%s/%s/%s_%s/changelog"
-
-
-class MyCache(apt.Cache):
-    def __init__(self, progress, rootdir=None):
-        apt.Cache.__init__(self, progress, rootdir)
-        # raise if we have packages in reqreinst state
-        # and let the caller deal with that (runs partial upgrade)
-        assert len(self.reqReinstallPkgs) == 0
-        # init the regular cache
-        self._initDepCache()
-        self.all_changes = {}
-        # on broken packages, try to fix via saveDistUpgrade()
-        if self._depcache.BrokenCount > 0:
-            self.saveDistUpgrade()
-        assert (self._depcache.BrokenCount == 0 and 
-                self._depcache.DelCount == 0)
-
-    def _initDepCache(self):
-        #apt_pkg.Config.Set("Debug::pkgPolicy","1")
-        #self.depcache = apt_pkg.GetDepCache(self.cache)
-        #self._depcache = apt_pkg.GetDepCache(self._cache)
-        self._depcache.ReadPinFile()
-        if os.path.exists(SYNAPTIC_PINFILE):
-            self._depcache.ReadPinFile(SYNAPTIC_PINFILE)
-        self._depcache.Init()
-    def clear(self):
-        self._initDepCache()
-    @property
-    def requiredDownload(self):
-        """ get the size of the packages that are required to download """
-        pm = apt_pkg.GetPackageManager(self._depcache)
-        fetcher = apt_pkg.GetAcquire()
-        pm.GetArchives(fetcher, self._list, self._records)
-        return fetcher.FetchNeeded
-    @property
-    def installCount(self):
-        return self._depcache.InstCount
-    def saveDistUpgrade(self):
-        """ this functions mimics a upgrade but will never remove anything """
-        self._depcache.Upgrade(True)
-        wouldDelete = self._depcache.DelCount
-        if self._depcache.DelCount > 0:
-            self.clear()
-        assert self._depcache.BrokenCount == 0 and self._depcache.DelCount == 0
-        self._depcache.Upgrade()
-        return wouldDelete
-    def matchPackageOrigin(self, pkg, matcher):
-        """ match 'pkg' origin against 'matcher', take versions between
-            installedVersion and candidateVersion into account too
-            Useful if installed pkg A v1.0 is available in both
-            -updates (as v1.2) and -security (v1.1). we want to display
-            it as a security update then
-        """
-        inst_ver = pkg._pkg.CurrentVer
-        cand_ver = self._depcache.GetCandidateVer(pkg._pkg)
-        # init with empty match
-        update_origin = matcher[(None,None)]
-        for ver in pkg._pkg.VersionList:
-            # discard is < than installed ver
-            if (inst_ver and
-                apt_pkg.VersionCompare(ver.VerStr, inst_ver.VerStr) <= 0):
-                #print "skipping '%s' " % ver.VerStr
-                continue
-            # check if we have a match
-            for(verFileIter,index) in ver.FileList:
-                if matcher.has_key((verFileIter.Archive, verFileIter.Origin)):
-                    indexfile = pkg._list.FindIndex(verFileIter)
-                    if indexfile: # and indexfile.IsTrusted:
-                        match = matcher[verFileIter.Archive, verFileIter.Origin]
-                        if match.importance > update_origin.importance:
-                            update_origin = match
-        return update_origin
-        
-    def get_changelog(self, name, lock):
-        # don't touch the gui in this function, it needs to be thread-safe
-        pkg = self[name]
-
-        # get the src package name
-        srcpkg = pkg.sourcePackageName
-
-        # assume "main" section 
-        src_section = "main"
-        # use the section of the candidate as a starting point
-        section = pkg._depcache.GetCandidateVer(pkg._pkg).Section
-
-        # get the source version, start with the binaries version
-        binver = pkg.candidateVersion
-        srcver = pkg.candidateVersion
-        #print "bin: %s" % binver
-
-        l = section.split("/")
-        if len(l) > 1:
-            src_section = l[0]
-
-        # lib is handled special
-        prefix = srcpkg[0]
-        if srcpkg.startswith("lib"):
-            prefix = "lib" + srcpkg[3]
-
-        # stip epoch, but save epoch for later when displaying the
-        # launchpad changelog
-        srcver_epoch = srcver
-        l = string.split(srcver,":")
-        if len(l) > 1:
-            srcver = "".join(l[1:])
-
-        try:
-            uri = CHANGELOGS_URI % (src_section,prefix,srcpkg,srcpkg, srcver)
-            # print "Trying: %s " % uri
-            changelog = urllib2.urlopen(uri)
-            #print changelog.read()
-            # do only get the lines that are new
-            alllines = ""
-            regexp = "^%s \((.*)\)(.*)$" % (re.escape(srcpkg))
-
-            i=0
-            while True:
-                line = changelog.readline()
-                if line == "":
-                    break
-                match = re.match(regexp,line)
-                if match:
-                    # strip epoch from installed version
-                    # and from changelog too
-                    installed = pkg.installedVersion
-                    if installed and ":" in installed:
-                        installed = installed.split(":",1)[1]
-                    changelogver = match.group(1)
-                    if changelogver and ":" in changelogver:
-                        changelogver = changelogver.split(":",1)[1]
-                    # we test for "==" here to ensure that the version
-                    # is actually really in the changelog - if not
-                    # just display it all, this catches cases like:
-                    # gcc-defaults with "binver=4.3.1" and srcver=1.76
-                    if (installed and 
-                        apt_pkg.VersionCompare(changelogver,installed)==0):
-                         break
-                # EOF (shouldn't really happen)
-                alllines = alllines + line
-
-            # Print an error if we failed to extract a changelog
-            if len(alllines) == 0:
-                alllines = _("The list of changes is not available")
-
-            # only write if we where not canceld
-            if lock.locked():
-                self.all_changes[name] = [alllines, srcpkg]
-        except urllib2.HTTPError, e:
-            if lock.locked():
-                self.all_changes[name] = [
-                    _("The list of changes is not available yet.\n\n"
-                      "Please use http://launchpad.net/ubuntu/+source/%s/%s/+changelog\n"
-                      "until the changes become available or try again "
-                      "later.") % (srcpkg, srcver_epoch),
-                    srcpkg]
-        except (IOError, httplib.BadStatusLine, socket.error), e:
-            print "caught exception: ", e
-            if lock.locked():
-                self.all_changes[name] = [_("Failed to download the list "
-                                            "of changes. \nPlease "
-                                            "check your Internet "
-                                            "connection."), srcpkg]
-        if lock.locked():
-            lock.release()
-
-class UpdateList:
-  class UpdateOrigin:
-    def __init__(self, desc, importance):
-      self.packages = []
-      self.importance = importance
-      self.description = desc
-
-  def __init__(self, parent):
-    # a map of packages under their origin
-    try:
-        pipe = os.popen("lsb_release -c -s")
-        dist = pipe.read().strip()
-        del pipe
-    except Exception, e:
-        print "Error in lsb_release: %s" % e
-        parent.error(_("Failed to detect distribution"),
-                     _("A error '%s' occurred while checking what system "
-                       "you are using.") % e)
-        sys.exit(1)
-    self.distUpgradeWouldDelete = 0
-    self.pkgs = {}
-    self.num_updates = 0
-    self.matcher = self.initMatcher(dist)
-    
-  def initMatcher(self, dist):
-      # (origin, archive, description, importance)
-      matcher_templates = [
-          ("%s-security" % dist, "Ubuntu", _("Important security updates"),10),
-          ("%s-updates" % dist, "Ubuntu", _("Recommended updates"), 9),
-          ("%s-proposed" % dist, "Ubuntu", _("Proposed updates"), 8),
-          ("%s-backports" % dist, "Ubuntu", _("Backports"), 7),
-          (dist, "Ubuntu", _("Distribution updates"), 6)
-      ]
-      matcher = {}
-      for (origin, archive, desc, importance) in matcher_templates:
-          matcher[(origin, archive)] = self.UpdateOrigin(desc, importance)
-      matcher[(None,None)] = self.UpdateOrigin(_("Other updates"), -1)
-      return matcher
-
-  def update(self, cache):
-    self.held_back = []
-
-    # do the upgrade
-    self.distUpgradeWouldDelete = cache.saveDistUpgrade()
-
-    dselect_upgrade_origin = self.UpdateOrigin(_("Previous selected"), 1)
-
-    # sort by origin
-    for pkg in cache:
-      if pkg.isUpgradable or pkg.markedInstall:
-        if pkg.candidateOrigin == None:
-            # can happen for e.g. locked packages
-            # FIXME: do something more sensible here (but what?)
-            print "WARNING: upgradable but no canidateOrigin?!?: ", pkg.name
-            continue
-        # check where the package belongs
-        origin_node = cache.matchPackageOrigin(pkg, self.matcher)
-        if not self.pkgs.has_key(origin_node):
-          self.pkgs[origin_node] = []
-        self.pkgs[origin_node].append(pkg)
-        self.num_updates = self.num_updates + 1
-      if pkg.isUpgradable and not (pkg.markedUpgrade or pkg.markedInstall):
-          self.held_back.append(pkg.name)
-    for l in self.pkgs.keys():
-      self.pkgs[l].sort(lambda x,y: cmp(x.name,y.name))
-    self.keepcount = cache._depcache.KeepCount
-
 
 class UpdateManagerDbusControler(dbus.service.Object):
     """ this is a helper to provide the UpdateManagerIFace """
@@ -336,7 +106,7 @@ class UpdateManagerDbusControler(dbus.service.Object):
 
 class UpdateManager(SimpleGladeApp):
 
-  def __init__(self, datadir):
+  def __init__(self, datadir, options):
     self.setupDbus()
     gtk.window_set_default_icon_name("update-manager")
 
@@ -348,7 +118,6 @@ class UpdateManager(SimpleGladeApp):
     self.window_main.set_sensitive(False)
     self.window_main.grab_focus()
     self.button_close.grab_focus()
-
     self.dl_size = 0
 
     # create text view
@@ -389,7 +158,7 @@ class UpdateManager(SimpleGladeApp):
       column_install.set_fixed_width(30)
       column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
       column.set_fixed_width(100)
-      self.treeview_update.set_fixed_height_mode(True)
+      self.treeview_update.set_fixed_height_mode(False)
 
     self.treeview_update.append_column(column_install)
     column_install.set_visible(True)
@@ -398,20 +167,48 @@ class UpdateManager(SimpleGladeApp):
     self.treeview_update.connect("button-press-event", self.show_context_menu)
     self.treeview_update.connect("row-activated", self.row_activated)
 
-
-
     # setup the help viewer and disable the help button if there
     # is no viewer available
-    self.help_viewer = HelpViewer("update-manager")
-    if self.help_viewer.check() == False:
-        self.button_help.set_sensitive(False)
+    #self.help_viewer = HelpViewer("update-manager")
+    #if self.help_viewer.check() == False:
+    #    self.button_help.set_sensitive(False)
+
+    if not os.path.exists("/usr/bin/software-properties-gtk"):
+        self.button_settings.set_sensitive(False)
 
     self.gconfclient = gconf.client_get_default()
     init_proxy(self.gconfclient)
-
+    # init show version
+    try:
+        self.show_versions = self.gconfclient.get_bool("/apps/update-manager/show_versions")
+    except gobject.GError, e:
+        self.show_versions = False
+    # keep track when we run (for update-notifier)
+    try:
+        self.gconfclient.set_int("/apps/update-manager/launch_time", int(time.time()))
+    except gobject.GError, e:
+        print "Error setting launch_time: ", e
+    # get progress object
+    self.progress = GtkProgress.GtkOpProgress(self.dialog_cacheprogress,
+                                              self.progressbar_cache,
+                                              self.label_cache,
+                                              self.window_main)
     # restore state
     self.restore_state()
+    # deal with no-focus-on-map
+    if options.no_focus_on_map:
+        self.window_main.set_focus_on_map(False)
+        self.progress._window.set_focus_on_map(False)
+    # show the main window
     self.window_main.show()
+    # it can only the iconified *after* it is shown (even if the docs
+    # claim otherwise)
+    if options.no_focus_on_map:
+        self.window_main.iconify()
+        self.window_main.set_urgency_hint(True)
+        self.window_main.connect("focus-in-event",
+                                 lambda w,e: (w.set_urgency_hint(False) and False))
+
 
   def install_column_view_func(self, cell_layout, renderer, model, iter):
     pkg = model.get_value(iter, LIST_PKG)
@@ -534,7 +331,7 @@ class UpdateManager(SimpleGladeApp):
       if self.expander_details.get_expanded():
         lock = thread.allocate_lock()
         lock.acquire()
-        t=thread.start_new_thread(self.cache.get_changelog,(name,lock))
+        t=thread.start_new_thread(self.cache.get_news_and_changelog,(name,lock))
         changes_buffer.set_text("%s\n" % _("Downloading list of changes..."))
         iter = changes_buffer.get_iter_at_line(1)
         anchor = changes_buffer.create_child_anchor(iter)
@@ -551,10 +348,15 @@ class UpdateManager(SimpleGladeApp):
         # download finished (or canceld, or time-out)
         button.hide()
         button.disconnect(id);
-
+    # display NEWS.Debian first, then the changelog
+    changes = ""
+    srcpkg = self.cache[name].sourcePackageName
+    if self.cache.all_news.has_key(name):
+        changes += self.cache.all_news[name]
     if self.cache.all_changes.has_key(name):
-      changes = self.cache.all_changes[name]
-      self.set_changes_buffer(changes_buffer, changes[0], name, changes[1])
+        changes += self.cache.all_changes[name]
+    if changes:
+        self.set_changes_buffer(changes_buffer, changes, name, srcpkg)
 
   def show_context_menu(self, widget, event):
     """
@@ -663,11 +465,15 @@ class UpdateManager(SimpleGladeApp):
           if self._get_last_apt_get_update_text() is not None:
               text_label_main = self._get_last_apt_get_update_text()
       else:
-          text_header = "<big><b>%s</b></big>" % \
-                        (ngettext("You can install %s update.",
-                                  "You can install %s updates.", 
-                                  num_updates) % \
-                                  num_updates)
+          # show different text on first run (UX team suggestion)
+          firstrun = self.gconfclient.get_bool("/apps/update-manager/first_run")
+          if firstrun:
+              text_header = "<big><b>%s</b></big>" % _("Welcome to Ubuntu")
+              text_label_main = _("These software updates have been issued since Ubuntu was released. If you don't want to install them now, choose \"Update Manager\" from the Administration Menu later.")
+              self.gconfclient.set_bool("/apps/update-manager/first_run", False)
+          else:
+              text_header = "<big><b>%s</b></big>" % _("Software updates are available for this computer")
+              text_label_main = _("If you don't want to install them now, choose \"Update Manager\" from the Administration menu later.")
           text_download = _("Download size: %s") % humanize_size(self.dl_size)
           self.notebook_details.set_sensitive(True)
           self.treeview_update.set_sensitive(True)
@@ -731,11 +537,46 @@ class UpdateManager(SimpleGladeApp):
     self.check_metarelease()
     self.invoke_manager(UPDATE)
 
-  def on_button_help_clicked(self, widget):
-    self.help_viewer.run()
+  #def on_button_help_clicked(self, widget):
+  #  self.help_viewer.run()
+
+  def on_button_settings_clicked(self, widget):
+      #print "on_button_settings_clicked"
+      try:
+          apt_pkg.PkgSystemUnLock()
+      except SystemError:
+          pass
+      cmd = ["/usr/bin/gksu", 
+             "--desktop", "/usr/share/applications/software-properties.desktop", 
+             "--", "/usr/bin/software-properties-gtk","--open-tab","2",
+             "--toplevel", "%s" % self.window_main.window.xid ]
+      self.window_main.set_sensitive(False)
+      p = subprocess.Popen(cmd)
+      while p.poll() is None:
+          while gtk.events_pending():
+              gtk.main_iteration()
+          time.sleep(0.05)
+      self.fillstore()
 
   def on_button_install_clicked(self, widget):
     #print "on_button_install_clicked"
+    err_sum = _("Not enough free disk space")
+    err_long= _("The upgrade needs a total of %s free space on disk '%s'. "
+                "Please free at least an additional %s of disk "
+                "space on '%s'. "
+                "Empty your trash and remove temporary "
+                "packages of former installations using "
+                "'sudo apt-get clean'.")
+    # check free space and error if its not enough
+    try:
+        self.cache.checkFreeSpace()
+    except NotEnoughFreeSpaceError, e:
+        for req in e.free_space_required_list:
+            self.error(err_sum, err_long % (req.size_total,
+                                            req.dir,
+                                            req.size_needed,
+                                            req.dir))
+        return
     self.invoke_manager(INSTALL)
     
   def invoke_manager(self, action):
@@ -771,7 +612,6 @@ class UpdateManager(SimpleGladeApp):
     self.window_main.window.set_cursor(None)
 
   def row_activated(self, treeview, path, column):
-      print self.list.held_back
       iter = self.store.get_iter(path)
       pkg = self.store.get_value(iter, LIST_PKG)
       origin = self.store.get_value(iter, LIST_ORIGIN)
@@ -918,7 +758,7 @@ class UpdateManager(SimpleGladeApp):
                    "new_version" : pkg.candidateVersion}
           else:
               version = _("Version %s") % pkg.candidateVersion
-          if self.gconfclient.get_bool("/apps/update-manager/show_versions"):
+          if self.show_versions:
               contents = "%s\n<small>%s %s</small>" % (contents, version, size)
           else:
               contents = "%s <small>%s</small>" % (contents, size)
@@ -956,13 +796,9 @@ class UpdateManager(SimpleGladeApp):
 
   def on_button_dist_upgrade_clicked(self, button):
       #print "on_button_dist_upgrade_clicked"
-      progress = GtkProgress.GtkFetchProgress(self,
-                                              _("Downloading the upgrade "
-                                                "tool"),
-                                              _("The upgrade tool will "
-                                                "guide you through the "
-                                                "upgrade process."))
-      fetcher = DistUpgradeFetcherGtk(new_dist=self.new_dist, parent=self, progress=progress)
+      fetcher = DistUpgradeFetcherGtk(new_dist=self.new_dist, parent=self, progress=GtkProgress.GtkFetchProgress(self))
+      if self.options.sandbox:
+          fetcher.run_options.append("--sandbox")
       fetcher.run()
       
   def new_dist_available(self, meta_release, upgradable_to):
@@ -994,15 +830,11 @@ class UpdateManager(SimpleGladeApp):
         #sys.exit()
 
     try:
-        progress = GtkProgress.GtkOpProgress(self.dialog_cacheprogress,
-                                             self.progressbar_cache,
-                                             self.label_cache,
-                                             self.window_main)
         if hasattr(self, "cache"):
-            self.cache.open(progress)
+            self.cache.open(self.progress)
             self.cache._initDepCache()
         else:
-            self.cache = MyCache(progress)
+            self.cache = MyCache(self.progress)
     except AssertionError:
         # if the cache could not be opened for some reason,
         # let the release upgrader handle it, it deals
@@ -1024,7 +856,7 @@ class UpdateManager(SimpleGladeApp):
         dialog.destroy()
         sys.exit(1)
     else:
-        progress.hide()
+        self.progress.hide()
 
   def check_auto_update(self):
       # Check if automatic update is enabled. If not show a dialog to inform
